@@ -63,6 +63,8 @@
 #include "ReniceDlg.h"
 #include "ui_ProcessWidgetUI.h"
 #include "scripting.h"
+#include "process_controller.h"
+#include "process_attribute.h"
 
 #include <sys/types.h>
 #include <unistd.h>
@@ -248,6 +250,8 @@ struct KSysGuardProcessListPrivate {
      */
     ProcessFilter mFilterModel;
 
+    KSysGuard::ProcessController *mProcessController = nullptr;
+
     /** The graphical user interface for this process list widget, auto-generated by Qt Designer */
     Ui::ProcessWidget *mUi;
 
@@ -298,6 +302,9 @@ KSysGuardProcessList::KSysGuardProcessList(QWidget* parent, const QString &hostN
     qRegisterMetaType<QList<long long> >();
     qDBusRegisterMetaType<QList<long long> >();
 
+    d->mProcessController = new KSysGuard::ProcessController(this);
+    d->mProcessController->setWidget(window());
+
     d->mUpdateIntervalMSecs = 0; //Set process to not update manually by default
     d->mUi->setupUi(this);
     d->mFilterModel.setSourceModel(&d->mModel);
@@ -340,6 +347,9 @@ KSysGuardProcessList::KSysGuardProcessList(QWidget* parent, const QString &hostN
     d->mUi->treeView->header()->hideSection(ProcessModel::HeadingIoRead);
     d->mUi->treeView->header()->hideSection(ProcessModel::HeadingIoWrite);
     d->mUi->treeView->header()->hideSection(ProcessModel::HeadingXMemory);
+    d->mUi->treeView->header()->hideSection(ProcessModel::HeadingCGroup);
+    d->mUi->treeView->header()->hideSection(ProcessModel::HeadingMACContext);
+    d->mUi->treeView->header()->hideSection(ProcessModel::HeadingVmPSS);
     // NOTE!  After this is all setup, the settings for the header are restored
     // from the user's last run.  (in restoreHeaderState)
     // So making changes here only affects the default settings.  To
@@ -363,6 +373,15 @@ KSysGuardProcessList::KSysGuardProcessList(QWidget* parent, const QString &hostN
     d->mUi->treeView->setDragEnabled(true);
     d->mUi->treeView->setDragDropMode(QAbstractItemView::DragOnly);
 
+    auto extraAttributes = d->mModel.extraAttributes();
+    for (int i = 0; i < extraAttributes.count(); ++i) {
+        auto attribute = extraAttributes.at(i);
+        if (attribute->isVisibleByDefault()) {
+            attribute->setEnabled(true);
+        } else {
+            d->mUi->treeView->header()->hideSection(ProcessModel::HeadingPluginStart + i);
+        }
+    }
 
     //Sort by username by default
     d->mUi->treeView->sortByColumn(ProcessModel::HeadingUser, Qt::AscendingOrder);
@@ -762,7 +781,7 @@ void KSysGuardProcessList::showColumnContextMenu(const QPoint &point){
                      d->mModel.ioInformation() == ProcessModel::SyscallsRate ||
                      d->mModel.ioInformation() == ProcessModel::ActualBytesRate;
 
-    if( index == ProcessModel::HeadingVmSize || index == ProcessModel::HeadingMemory || index == ProcessModel::HeadingXMemory || index == ProcessModel::HeadingSharedMemory || ( (index == ProcessModel::HeadingIoRead || index == ProcessModel::HeadingIoWrite) && d->mModel.ioInformation() != ProcessModel::Syscalls)) {
+    if( index == ProcessModel::HeadingVmSize || index == ProcessModel::HeadingMemory || index == ProcessModel::HeadingXMemory || index == ProcessModel::HeadingSharedMemory || index == ProcessModel::HeadingVmPSS || ( (index == ProcessModel::HeadingIoRead || index == ProcessModel::HeadingIoWrite) && d->mModel.ioInformation() != ProcessModel::Syscalls)) {
         //If the user right clicks on a column that contains a memory size, show a toggle option for displaying
         //the memory in different units.  e.g.  "2000 k" or "2 m"
         menu.addSeparator()->setText(i18n("Display Units"));
@@ -958,10 +977,18 @@ void KSysGuardProcessList::showColumnContextMenu(const QPoint &point){
 
     int i = result->data().toInt();
     //We set data to be negative to hide a column, and positive to show a column
-    if(i < 0)
-        d->mUi->treeView->hideColumn(-1-i);
-    else {
+    if(i < 0) {
+        auto index = -1 - i;
+        d->mUi->treeView->hideColumn(index);
+
+        if (index >= ProcessModel::HeadingPluginStart) {
+            d->mModel.extraAttributes().at(index - ProcessModel::HeadingPluginStart)->setEnabled(false);
+        }
+    } else {
         d->mUi->treeView->showColumn(i);
+        if (i >= ProcessModel::HeadingPluginStart) {
+            d->mModel.extraAttributes().at(i - ProcessModel::HeadingPluginStart)->setEnabled(true);
+        }
         updateList();
         d->mUi->treeView->resizeColumnToContents(i);
         d->mUi->treeView->resizeColumnToContents(d->mFilterModel.columnCount());
@@ -1033,6 +1060,12 @@ void KSysGuardProcessList::hideEvent ( QHideEvent * event )  //virtual protected
     //stop any scripts running, to save on memory
     if(d->mScripting)
         d->mScripting->stopAllScripts();
+
+    //Disable all plugin-provided attributes
+    for (auto attribute : d->mModel.extraAttributes()) {
+        attribute->setEnabled(false);
+    }
+
     QWidget::hideEvent(event);
 }
 
@@ -1042,6 +1075,14 @@ void KSysGuardProcessList::showEvent ( QShowEvent * event )  //virtual protected
     updateList();
     QHeaderView *header = d->mUi->treeView->header();
     d->mUi->treeView->sortByColumn(header->sortIndicatorSection(), header->sortIndicatorOrder());
+
+    auto attributes = d->mModel.extraAttributes();
+    for (int i = 0; i < attributes.count(); ++i) {
+        if (!header->isSectionHidden(ProcessModel::HeadingPluginStart + i)) {
+            attributes.at(i)->setEnabled(true);
+        }
+    }
+
     QWidget::showEvent(event);
 }
 
@@ -1126,29 +1167,13 @@ void KSysGuardProcessList::setUpdateIntervalMSecs(int intervalMSecs)
 
 bool KSysGuardProcessList::reniceProcesses(const QList<long long> &pids, int niceValue)
 {
-    QList< long long> unreniced_pids;
-    for (int i = 0; i < pids.size(); ++i) {
-        bool success = d->mModel.processController()->setNiceness(pids.at(i), niceValue);
-        if(!success) {
-            unreniced_pids << pids.at(i);
-        }
-    }
-    if(unreniced_pids.isEmpty()) return true; //All processes were reniced successfully
-    if(!d->mModel.isLocalhost()) return false; //We can't use kauth to renice non-localhost processes
-
-
-    KAuth::Action action(QStringLiteral("org.kde.ksysguard.processlisthelper.renice"));
-    action.setParentWidget(window());
-    d->setupKAuthAction( action, unreniced_pids);
-    action.addArgument(QStringLiteral("nicevalue"), niceValue);
-    KAuth::ExecuteJob *job  = action.execute();
-
-    if (job->exec()) {
+    auto result = d->mProcessController->setPriority(pids, niceValue);
+    if (result == KSysGuard::ProcessController::Result::Success) {
         updateList();
-    } else if (!job->exec()) {
+        return true;
+    } else if (result == KSysGuard::ProcessController::Result::Error) {
         KMessageBox::sorry(this, i18n("You do not have the permission to renice the process and there "
-                    "was a problem trying to run as root.  Error %1 %2", job->error(), job->errorString()));
-        return false;
+                                      "was a problem trying to run as root."));
     }
     return true;
 }
@@ -1286,95 +1311,44 @@ void KSysGuardProcessList::reniceSelectedProcesses()
 
 bool KSysGuardProcessList::changeIoScheduler(const QList< long long> &pids, KSysGuard::Process::IoPriorityClass newIoSched, int newIoSchedPriority)
 {
-    if(newIoSched == KSysGuard::Process::None) newIoSched = KSysGuard::Process::BestEffort;
-    if(newIoSched == KSysGuard::Process::Idle) newIoSchedPriority = 0;
-    QList< long long> unchanged_pids;
-    for (int i = 0; i < pids.size(); ++i) {
-        bool success = d->mModel.processController()->setIoNiceness(pids.at(i), newIoSched, newIoSchedPriority);
-        if(!success) {
-            unchanged_pids << pids.at(i);
-        }
-    }
-    if(unchanged_pids.isEmpty()) return true;
-    if(!d->mModel.isLocalhost()) return false; //We can't use kauth to affect non-localhost processes
-
-    KAuth::Action action(QStringLiteral("org.kde.ksysguard.processlisthelper.changeioscheduler"));
-    action.setParentWidget(window());
-
-    d->setupKAuthAction( action, unchanged_pids);
-    action.addArgument(QStringLiteral("ioScheduler"), (int)newIoSched);
-    action.addArgument(QStringLiteral("ioSchedulerPriority"), newIoSchedPriority);
-
-    KAuth::ExecuteJob *job  = action.execute();
-
-    if (job->exec()) {
+    auto result = d->mProcessController->setIOScheduler(pids, newIoSched, newIoSchedPriority);
+    if (result == KSysGuard::ProcessController::Result::Success) {
         updateList();
-    } else if (!job->exec()) {
+        return true;
+    } else if (result == KSysGuard::ProcessController::Result::Error) {
         KMessageBox::sorry(this, i18n("You do not have the permission to change the I/O priority of the process and there "
-                    "was a problem trying to run as root.  Error %1 %2", job->error(), job->errorString()));
-        return false;
+                                      "was a problem trying to run as root."));
     }
-    return true;
+
+    return false;
 }
 
 bool KSysGuardProcessList::changeCpuScheduler(const QList< long long> &pids, KSysGuard::Process::Scheduler newCpuSched, int newCpuSchedPriority)
 {
-    if(newCpuSched == KSysGuard::Process::Other || newCpuSched == KSysGuard::Process::Batch) newCpuSchedPriority = 0;
-    QList< long long> unchanged_pids;
-    for (int i = 0; i < pids.size(); ++i) {
-        bool success = d->mModel.processController()->setScheduler(pids.at(i), newCpuSched, newCpuSchedPriority);
-        if(!success) {
-            unchanged_pids << pids.at(i);
-        }
-    }
-    if(unchanged_pids.isEmpty()) return true;
-    if(!d->mModel.isLocalhost()) return false; //We can't use KAuth to affect non-localhost processes
+    auto result = d->mProcessController->setCPUScheduler(pids, newCpuSched, newCpuSchedPriority);
 
-    KAuth::Action action(QStringLiteral("org.kde.ksysguard.processlisthelper.changecpuscheduler"));
-    action.setParentWidget(window());
-    d->setupKAuthAction( action, unchanged_pids);
-    action.addArgument(QStringLiteral("cpuScheduler"), (int)newCpuSched);
-    action.addArgument(QStringLiteral("cpuSchedulerPriority"), newCpuSchedPriority);
-    KAuth::ExecuteJob *job  = action.execute();
-
-    if (job->exec()) {
+    if (result == KSysGuard::ProcessController::Result::Success) {
         updateList();
-    } else if (!job->exec()) {
+        return true;
+    } else if (result == KSysGuard::ProcessController::Result::Error) {
         KMessageBox::sorry(this, i18n("You do not have the permission to change the CPU Scheduler for the process and there "
-                    "was a problem trying to run as root.  Error %1 %2", job->error(), job->errorString()));
-        return false;
+                                      "was a problem trying to run as root."));
     }
-    return true;
+    return false;
 }
 
 bool KSysGuardProcessList::killProcesses(const QList< long long> &pids, int sig)
 {
-    QList< long long> unkilled_pids;
-    for (int i = 0; i < pids.size(); ++i) {
-        bool success = d->mModel.processController()->sendSignal(pids.at(i), sig);
-        // If we failed due to a reason other than insufficient permissions, elevating to root can't
-        // help us
-        if(!success && (d->mModel.processController()->lastError() == KSysGuard::Processes::InsufficientPermissions || d->mModel.processController()->lastError() == KSysGuard::Processes::Unknown)) {
-            unkilled_pids << pids.at(i);
-        }
-    }
-    if(unkilled_pids.isEmpty()) return true;
-    if(!d->mModel.isLocalhost()) return false; //We can't elevate privileges to kill non-localhost processes
+    auto result = d->mProcessController->sendSignal(pids, sig);
 
-    KAuth::Action action(QStringLiteral("org.kde.ksysguard.processlisthelper.sendsignal"));
-    action.setParentWidget(window());
-    d->setupKAuthAction( action, unkilled_pids);
-    action.addArgument(QStringLiteral("signal"), sig);
-    KAuth::ExecuteJob *job  = action.execute();
-
-    if (job->exec()) {
+    if (result == KSysGuard::ProcessController::Result::Success) {
         updateList();
-    } else if (!job->exec()) {
+        return true;
+    } else if (result == KSysGuard::ProcessController::Result::Error) {
         KMessageBox::sorry(this, i18n("You do not have the permission to kill the process and there "
-                    "was a problem trying to run as root.  Error %1 %2", job->error(), job->errorString()));
-        return false;
+                                      "was a problem trying to run as root."));
     }
-    return true;
+    return false;
 }
 
 void KSysGuardProcessList::killSelectedProcesses()
